#!/usr/bin/perl -I/usr/sausalito/perl
# $Id: vsite_apop.pl,v 1.1.2.3 2002/02/02 03:39:31 pbaltz Exp $
# Copyright 2002 Sun Microsystems, Inc., All rights reserved.
#
# This handler maintains the user entries in the apop database.
# It handles disabling/enabling apop at the site level.
#

use CCE;
use GDBM_File;

# Globals.
my $PopAuth = '/etc/pop.auth';

my $cce = new CCE(Domain => 'base-email');
$cce->connectfd();

# make sure pop auth db exists and is properly owned
if (! -f $PopAuth) {
	system('/usr/sbin/popauth -init');
	if ($?) {
		$cce->bye('FAIL');
		exit(1);
	}
}

my $vsite = $cce->event_object();
my ($ok, $apop) = $cce->get($cce->event_oid(), 'APOP');

if (!$ok) {
	$cce->warn('[[base-email.cantReadAPOP]]');
	$cce->bye('FAIL');
	exit(1);
}

# see if there are any users using apop
my @apop_uoids = $cce->find('User',
			    { 'site' => $vsite->{name}, 'APOP.enabled' => 1 });
if (scalar(@apop_uoids) == 0) {
	# no users with apop enabled so just leave successfully
	$cce->bye('SUCCESS');
	exit(0);
}

#
# otherwise need to sync the database of pop users so that users for this
# site with apop enabled get added/removed when apop is enabled/disabled
# for the site
#
my %db;
Sauce::Util::modifyfile($PopAuth);
if(!tie(%db, 'GDBM_File', $PopAuth, GDBM_WRITER, 0600)) {
	$cce->warn('could_not_tie_apop $!');
	$cce->bye('FAIL');
	exit 1;
}

# iterate all users with APOP enabled
for my $uoid (@apop_uoids) {
	($ok, my $user) = $cce->get($uoid);
	($ok, my $user_apop) = $cce->get($uoid, 'APOP');
	if (!$ok) {
		$cce->bye('FAIL', 'cantGetUser');
		exit(1);
	}

	if ($apop->{enabled}) {
		# add user to apop db
		$db{ $user->{name} . chr(0) } = $user_apop->{apop_password} .
							chr(0) . chr(0);
	} else {
		# remove user from apop db
		delete $db{ $user->{name} . chr(0) };
	}
}

untie(%db);
$cce->bye('SUCCESS');
exit(0);
